/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  3DFile.h
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#ifndef _3DFILE_H_
#define _3DFILE_H_
#include "KonStructs.h"
#include <iostream>
#include <string>
#include <vector>
#include <cstring>

class C3DFile  
{

public:
    C3DFile();
    C3DFile(const C3DFile& c3dfile);
    virtual ~C3DFile();
    
    void SetNVerts(unsigned long ulNVerts){ m_ulNVerts = ulNVerts; }
    void SetNFaces(unsigned long ulNFaces){ m_ulNFaces = ulNFaces; }
    void SetVerts(VRTX* pvrtx);
    void SetVerts(VRTX** pvrtx, int nWidth, int nHeight);
    void SetFaces(std::vector<TRNGL>& vTrngls);
    void SetFaces(TRNGL* ptrngls);
    void Alloc();
    void SetComment(char* pcCmnt) { 
	m_strCmnt = pcCmnt;
    }
    void SetMatrix(const float* pfM, bool bTr = false){
	if(m_pfMat == NULL){
	    m_pfMat = new float[16];
	}
	if(bTr){
	    for(int i = 0; i < 4; i++){
		for(int j = 0; j < 4; j++){
		    m_pfMat[i*4+j] = pfM[j*4+i];
		}
	    }
	}else{
	    memcpy(m_pfMat, pfM, sizeof(float)*16);
	}
    }
    void Transform(const float* pfM, bool bT = false);

    VRTX* GetVerts() const { return m_pVerts; }
    TRNGL* GetFaces() const{ return m_pFaces; }
    unsigned long GetNVerts() const { return m_ulNVerts; }
    unsigned long GetNFaces() const { return m_ulNFaces; }
    char* GetShortFName() const { return m_pcShortFName; }
    char* GetFName() const { return m_pcFName; }
    void GetAvgNormal(float* pfAvgN) const { 
	pfAvgN[0] = m_d3AvgN.dx; pfAvgN[1] = m_d3AvgN.dy; pfAvgN[2] = m_d3AvgN.dz;
    }
    void GetCentroid(float* pfCntrd) const {
	pfCntrd[0] = m_d3Cntrd.dx; pfCntrd[1] = m_d3Cntrd.dy; pfCntrd[2] = m_d3Cntrd.dz;
    }
    void GetMinMax(float& pfMinX, float& pfMaxX, float& pfMinY, float& pfMaxY,
		   float& pfMinZ, float& pfMaxZ) const {
	pfMinX = m_fMinX; pfMaxX = m_fMaxX;
	pfMinY = m_fMinY; pfMaxY = m_fMaxY;
	pfMinZ = m_fMinZ; pfMaxZ = m_fMaxZ;
    }
    void GetLong(float& pfLong) const {
	pfLong = m_fLong;
    }
    
    int WritePly(const char* pcFName, float fScale, bool bAscii = true, bool bLittle = false);
    int WriteX(const char* pcFName, float fScale, bool bFlip, bool bAscii = true, bool bLittle = false);
    int ReadPly(const char* pcFName, float fScale = 10.0, bool bReverse = false);
    int ReadFile(const char* pcFName, float fScale = 10.0, bool bReverse = false);
    
    int CheckBoundary();
    int ComputeNormals();
    int ComputeCentroid();
    int ComputeMinMax();
    
private:
    unsigned long m_ulNVerts, m_ulNFaces;
    VRTX* m_pVerts;
    TRNGL* m_pFaces;
    char* m_pcShortFName;
    char* m_pcFName;
    D3DPoint m_d3AvgN, m_d3Cntrd;
    float m_fMinX, m_fMinY, m_fMinZ, m_fMaxX, m_fMaxY, m_fMaxZ;
    float m_fLong;
    std::string m_strCmnt;
    float* m_pfMat;
};

#endif // _3DFile_H_


