/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  KDTree.h
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

/* 
 * Currently all source code is adapted from Mark's original `knn' source code.
 */

#ifndef _KDTREE_H_
#define _KDTREE_H_
#include <iostream>
#include <string.h>
#include "KDTreeStruct.h"

struct cntnt{
    int id;
    unsigned long size;
    int dim;
    float* data;
    float* normal;
    float* inten;
    bool* bound;
};
typedef struct cntnt CNTNT;

class CKDTree  
{
public:
    CKDTree();
    CKDTree(int nNRoots){ 
	Allocate(nNRoots);
    }
    virtual ~CKDTree();
    void Allocate(int nNRoots){
	m_nNRoots = nNRoots;
	m_pContents = new (CNTNT (*[nNRoots]));
	KDNNRoot = new (BUCKET (*[nNRoots])); 
    }
    int GetSize() const { return m_nNRoots; }
    void MakeKDTree(BUCKET *root, float data[], register int k, register int n);
    void SetContent(int ncnt, int nid, unsigned long ulsize, int ndim, 
		    float* pfdata, float* pfnormal, float* pfinten, bool* pbbound){
	m_pContents[ncnt] = new CNTNT;
	m_pContents[ncnt]->id = nid;
	m_pContents[ncnt]->size = ulsize;
	m_pContents[ncnt]->dim = ndim;
	m_pContents[ncnt]->data = new float[m_pContents[ncnt]->size*m_pContents[ncnt]->dim];
	m_pContents[ncnt]->normal = new float[m_pContents[ncnt]->size*m_pContents[ncnt]->dim];
	m_pContents[ncnt]->inten = new float[m_pContents[ncnt]->size];
	m_pContents[ncnt]->bound = new bool[m_pContents[ncnt]->size];
	memcpy(m_pContents[ncnt]->data, pfdata, sizeof(float)*m_pContents[ncnt]->size*m_pContents[ncnt]->dim);
	memcpy(m_pContents[ncnt]->normal, pfnormal, sizeof(float)*m_pContents[ncnt]->size*m_pContents[ncnt]->dim);	
	memcpy(m_pContents[ncnt]->inten, pfinten, sizeof(float)*m_pContents[ncnt]->size);	
	memcpy(m_pContents[ncnt]->bound, pbbound, sizeof(bool)*m_pContents[ncnt]->size);	
    }
    
    CNTNT* GetContent(int ncnt){
	return m_pContents[ncnt];
    }
    
    
    void KNNSearch(float data[], float key[], float pqd[], int pqr[], int k, int m, int n, int i, bool bCheck, int nstop, int& nresult);
    BUCKET* GetRoot(int nID){ return KDNNRoot[nID]; }
    void AllocRoot(int nID){ KDNNRoot[nID] = new BUCKET; }
    void DeleteRoot(int nID){ delete KDNNRoot[nID]; }
    void DeleteAllRoots(){ for(int i = 0; i < m_nNRoots; i++) DeleteRoot(i); }
    void DeleteContents(){ 
	for(int i = 0; i < m_nNRoots; i++){
	    delete [] m_pContents[i]->data;
	    delete [] m_pContents[i]->normal;
	    delete [] m_pContents[i]->inten;
	    delete [] m_pContents[i]->bound;
	    delete m_pContents[i];
	}
	delete [] m_pContents;
    }
    void FreeAll(){ 
	for(int i = 0; i < m_nNRoots; i++) NewKNNData(i); 
//	DeleteAllRoots(); // already done in NewKNNData
	DeleteContents();
    }
    
    int m_nNRoots;
    CNTNT **m_pContents;
    
    BUCKET **KDNNRoot;
    
private:
    
    void BuildKDTree(BUCKET *result,float data[], int index[], int k, int n);
    float KDMedian(int d, float *data, int index[], int k, int n);
    float KDSpreadEstimate(int d, float *data, int index[], int k, int n);
    BUCKET_PTR KDTreeSearchNode(BUCKET *node, float key[], int k);
    void KDTreeInsertItem(float data[], BUCKET *root, float key[], 
			  int index, int k);
    void KDTreeRemoveItem(BUCKET *root, float key[], int index, int k);
    
    void KDNNSearch(BUCKET *root, float data[], float key[], float pqd[], int pqr[], int k, 
		    int m, bool bCheck, int n, int nstep, int& nresult);
    void KDNNSearchNode(BUCKET *node, float data[], float key[], float pqd[], 
			int pqr[], int k, int m, int n, int& ncnt, int nstop, int& nresult);
    
    void NewKNNData(int i);
    void freeKNNTree(BUCKET *node);
    void KNNInsertItem(float data[], float key[], int k, int index, int i);
    void KNNRemoveItem(float key[], int k, int index, int i);
    
    void mdian3(float x[], int n, float *xmed);
    void sort(int n, float ra[]);
};

#endif /* _KDTREE_H_ */
