/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  KDTreeStruct.h
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */
/*
 * Adapted from Mark Wheeler's kd-tree.h.
 * Original comment attached below. 
 */

/* 
 * kd-tree.h 
 * 
 * Created 5-27-92 by Mark Wheeler at Carnegie Mellon University, 
 *                    mdwheel@cs.cmu.edu
 * 
 * definitions code to build a kd-tree which 
 * stores a set of n k-dimensional vectors
 * with a uniform dissimilarity metric
 * 
 * Algorithm taken from:
 * "An Algorithm for Finding Best Matches in Logarithmic Expected Time"
 *        by Jerome H. Friedman, Jon Bentley and Raphael Finkel
 * found in "ACM Transactions on Mathematical Software"
 *             VOLUME 3 No. 3 1977, pp. 209-226 * Created 5-27-92 by MD Wheeler
 * 
 *
 * Modified 5-15-93 by David Simon - Optimized for 3d trees. It is assumed
 * that add data points are 3 dimensional. Also added register variables. 
 *
 */

#ifndef _KDTreeStruct_H_
#define _KDTreeStruct_H_ 

#define TERMINAL 1
#define NONTERMINAL 0
#define BUCKET_SIZE 5
//#define NULL 0


/*
#define CoordinateDistance(d,x,y,r) {\
  float val = ((x)[(d)] - (y)[(d)]);\
  (r) = val*val;\
}
*/
#define CoordinateDistance(d,x,y,r) { float val = ((x)[(d)] - (y)[(d)]); (r) = val*val; }


#define Dissimilarity(x) (x)

struct bucket {
    int type;
    int dimension;
    float median;
    struct bucket *left,*right;
    int *index;
    int n;
};

typedef struct bucket BUCKET;
typedef struct bucket *BUCKET_PTR;

#endif /* KDTreeStruct_h */
