/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  KFXGLView.cpp
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#include "KDefs.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <iostream>
#include "KFXGLView.h"
#include "PSRegWin.h"
#include "trackball.h"

#define TRANS_OBJECT_SCALE 10.0
#define TRANS_VIEW_SCALE 10.0
#define CI_ANGLE 45.0
#define CI_SCALE 1.0

FXDEFMAP(CKFXGLView) CKFXGLViewMap[] = {
    FXMAPFUNC(SEL_PAINT, 0, CKFXGLView::onPaint),
    FXMAPFUNC(SEL_COMMAND, CKFXGLView::ID_CAMERA, CKFXGLView::onCmdCamera),
    FXMAPFUNC(SEL_UPDATE, CKFXGLView::ID_CAMERA, CKFXGLView::onUpdCamera),
    FXMAPFUNC(SEL_COMMAND, CKFXGLView::ID_OBJECT, CKFXGLView::onCmdObject),
    FXMAPFUNC(SEL_UPDATE, CKFXGLView::ID_OBJECT, CKFXGLView::onUpdObject),
    FXMAPFUNC(SEL_COMMAND, CKFXGLView::ID_POINTS, CKFXGLView::onCmdPoints),
    FXMAPFUNC(SEL_UPDATE, CKFXGLView::ID_POINTS, CKFXGLView::onUpdPoints),
    FXMAPFUNC(SEL_COMMAND, CKFXGLView::ID_WIREFRAME, CKFXGLView::onCmdWireframe),
    FXMAPFUNC(SEL_UPDATE, CKFXGLView::ID_WIREFRAME, CKFXGLView::onUpdWireframe),
    FXMAPFUNC(SEL_COMMAND, CKFXGLView::ID_SURFACE, CKFXGLView::onCmdSurface),
    FXMAPFUNC(SEL_UPDATE, CKFXGLView::ID_SURFACE, CKFXGLView::onUpdSurface),
    FXMAPFUNC(SEL_COMMAND, CKFXGLView::ID_DUMP, CKFXGLView::onCmdDumpImage),
    FXMAPFUNC(SEL_COMMAND, CKFXGLView::ID_COLOR, CKFXGLView::onCmdColor),
    FXMAPFUNC(SEL_UPDATE, CKFXGLView::ID_COLOR, CKFXGLView::onUpdColor),
    FXMAPFUNC(SEL_LEFTBUTTONPRESS, 0, CKFXGLView::onLeftBtnPress),
    FXMAPFUNC(SEL_LEFTBUTTONRELEASE, 0, CKFXGLView::onLeftBtnRelease),
    FXMAPFUNC(SEL_RIGHTBUTTONPRESS, 0, CKFXGLView::onRightBtnPress),
    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, 0, CKFXGLView::onRightBtnRelease),
    FXMAPFUNC(SEL_MOTION, 0, CKFXGLView::onMouseMove),
    
};

FXIMPLEMENT(CKFXGLView, FXGLCanvas, CKFXGLViewMap, ARRAYNUMBER(CKFXGLViewMap))


CKFXGLView::CKFXGLView()
{

}

CKFXGLView::CKFXGLView(FXComposite* p, FXGLVisual* vis, FXObject* tgt, \
		       FXSelector sel, FXuint opts, \
		       FXint x, FXint y, FXint w, FXint h)
//    : FXGLCanvas(p, vis, NULL, tgt, sel, opts, x, y, w, h)
    : FXGLCanvas(p, vis, tgt, sel, opts, x, y, w, h)
{
    initialize();
}

CKFXGLView::CKFXGLView(FXComposite* p,FXGLVisual* vis, \
		       FXGLViewer* sharegroup, \
		       FXObject* tgt, FXSelector sel, FXuint opts, \
		       FXint x, FXint y, FXint w, FXint h)
    : FXGLCanvas(p, vis, sharegroup, tgt, sel, opts, x, y, w, h)
{
    initialize();
}

CKFXGLView::~CKFXGLView()
{
    delete [] m_qObj;
}


void CKFXGLView::initialize()
{
    m_nVWidth = m_nVHeight = 0;
    m_fvEye[0] = 12.0; m_fvEye[1] = 32.0; m_fvEye[2] = 18.0;
    m_fvCenter[0] = m_fvCenter[1] = m_fvCenter[2] = 0.0;
    m_fvUp[0] = 0.0; m_fvUp[1] = 1.0; m_fvUp[2] = 0.0;
    m_fAngle = 25.0;
    m_fNear = (float)0.1; m_fFar = (float)1000.0;
    m_bProj = false;
    setIdentityMat(m_pfRotMat);
    setIdentityMat(m_pfTransMat);
    m_fScale = 1.0;
    m_fxuiManipulate = FREEZE;
    m_fxuiRenderState = POINTS;
    m_fxuiColor = NOCOLOR;

    m_qObj = new float[4];
    m_qObj[0] = m_qObj[1] = m_qObj[2] = 0.0; m_qObj[3] = 1.0;
}

// Create window
void CKFXGLView::create()
{
    FXASSERT(sizeof(FXuint)==sizeof(GLuint));
    
    // Create Window
    FXGLCanvas::create();
    
    // Set up OpenGL environment
    glsetup();
    
    update();
}

// Set up GL context
void CKFXGLView::glsetup()
{
    // Make GL context current
    if(makeCurrent()){
	// Set up the viewport
	glViewport(0, 0, 800, 600);
	// Set up the matrices
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	gluPerspective(m_fAngle, (GLfloat)800/600, m_fNear, m_fFar);
	gluLookAt(m_fvEye[0], m_fvEye[1], m_fvEye[2],
		  m_fvCenter[0], m_fvCenter[1], m_fvCenter[2],
		  m_fvUp[0], m_fvUp[1], m_fvUp[2]);

	// Initialize GL context
	glRenderMode(GL_RENDER);
	
	// Fast hints
	glHint(GL_POLYGON_SMOOTH_HINT,GL_FASTEST);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_FASTEST);
	glHint(GL_FOG_HINT,GL_FASTEST);
	glHint(GL_LINE_SMOOTH_HINT,GL_FASTEST);
	glHint(GL_POINT_SMOOTH_HINT,GL_FASTEST);

	// Z-buffer test on
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glDepthRange(0.0,1.0);
	glClearDepth(1.0);
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	
	glEnable(GL_CULL_FACE);
	glFrontFace(GL_CW);
	glCullFace(GL_BACK);

	makeNonCurrent();
    }
}

// Event handling funcs.

long CKFXGLView::onPaint(FXObject*, FXSelector, void*)
{
    FXGLVisual* fxglvis = (FXGLVisual *)getVisual();
    if(makeCurrent()){
	drawScene();
	if(fxglvis->isDoubleBuffer()) swapBuffers();
	makeNonCurrent();
    }

    return 1;
}
    

void CKFXGLView::drawScene()
{
    // Set viewport
    if(m_nVWidth != getWidth() || m_nVHeight != getHeight()){ // size change
	m_nVWidth = getWidth();
	m_nVHeight = getHeight();
	glViewport(0, 0, m_nVWidth, m_nVHeight);
	m_bProj = false; // require a re-set up of perspective
#ifdef DTLDEBUG
	printViewport();
#endif // DTLDEBUG
    }

    if(!m_bProj){
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(m_fAngle, (GLfloat)m_nVWidth/(GLfloat)m_nVHeight, m_fNear, m_fFar);
	gluLookAt(m_fvEye[0], m_fvEye[1], m_fvEye[2],
		  m_fvCenter[0], m_fvCenter[1], m_fvCenter[2],
		  m_fvUp[0], m_fvUp[1], m_fvUp[2]);
	m_bProj = true;
#ifdef DTLDEBUG
	printProjMat();
	printMViewMat();
#endif // DTLDEBUG
    }
    
    glEnable(GL_CULL_FACE);
    glFrontFace(GL_CW);
    glCullFace(GL_BACK);    
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    // Clear background
    clearVPort();
    glMatrixMode(GL_MODELVIEW);    
    if(m_vObjs.size() != 0){
	std::vector<SROBJ*>::iterator itr;
	int nID(0);
	glPushMatrix();
	glLoadIdentity();
	glMultMatrixf(m_pfRotMat); // rotate the view
	glMultMatrixf(m_pfTransMat); // translate the view
	glScalef(m_fScale, m_fScale, m_fScale); // zoom the view
	makeGrid();
	for(itr = m_vObjs.begin(); itr != m_vObjs.end(); itr++){
	    glPushMatrix();
	    glMultMatrixf(m_vmatModel[nID]);// rotate the model itself
	    renderSRObject(*itr, nID, m_fxuiRenderState, (m_fxuiColor == COLOR ? true : false));
	    glPopMatrix();
	    glFlush();
	    nID++;
	}
	glPopMatrix();

    }else{ // there's nothing to draw

    }
    glFlush();

}

long CKFXGLView::onMouseMove(FXObject*, FXSelector, void* ptr)
{
    /*
      View && Left -> Rotate view point
      View && Right -> Translate view point
      Model && Left -> Check which model and rotate that model
      Model && Right -> Check which model and rotate that model
     */
    FXEvent* ev = (FXEvent*)ptr;
    NPoint npLast, npCurr;
    npLast.nx = ev->last_x;
    npLast.ny = ev->last_y;
    npCurr.nx = ev->win_x;
    npCurr.ny = ev->win_y;
    
    switch(m_fxuiManipulate){
    case CAMERA: // view
	if(ev->state & LEFTBUTTONMASK){
	    rotateView(npLast, npCurr);
	}else if(ev->state & RIGHTBUTTONMASK){ 
	    if(ev->state & SHIFTMASK){
		zoomView(npLast, npCurr);
	    }else{
		transView(npLast, npCurr);
	    }
	}else{
	    // nop 
	}
	break;

    case OBJECT: // model 
	if(ev->state & LEFTBUTTONMASK){ // rotate
	    for(int i = 0; i < m_vObjs.size(); i++){
		if(m_vTItems[i]->isSelected()){ // check which one to rotate
		    rotateObject(i, npLast, npCurr);
		}
	    }
	}else if(ev->state & RIGHTBUTTONMASK){ // translate
	    if(ev->state & SHIFTMASK){ // z
		for(int i = 0; i < m_vObjs.size(); i++){
		    if(m_vTItems[i]->isSelected()){ // check which one to rotate
			transObject(i, npLast, npCurr, true);
		    }
		}		
	    }else{
		for(int i = 0; i < m_vObjs.size(); i++){
		    if(m_vTItems[i]->isSelected()){ // check which one to rotate
			transObject(i, npLast, npCurr, false);
		    }
		}
	    }
	}
	break;

    case FREEZE: // don't move anything
	// nop
	break;

    default:
	// nop
	break;
    }    
    // update view
    callonPaint();

    return 1;
}

long CKFXGLView::onCmdCamera(FXObject*, FXSelector, void*)
{
    m_fxuiManipulate = m_fxuiManipulate == CAMERA ? FREEZE : CAMERA;

    return 1;
}


long CKFXGLView::onUpdCamera(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg = m_fxuiManipulate == CAMERA ? ID_CHECK : ID_UNCHECK;
    sender->handle(this, MKUINT(ID_SHOW, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(ID_ENABLE, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(msg, SEL_COMMAND), NULL);
    FXButton* fxb = (FXButton*)sender;
    if(msg == ID_CHECK){
	fxb->setState(STATE_DOWN);
    }else{
	fxb->setState(STATE_UP);
    }

    return 1;
}


long CKFXGLView::onCmdObject(FXObject*, FXSelector, void*)
{
    m_fxuiManipulate = m_fxuiManipulate == OBJECT ? FREEZE : OBJECT;

    return 1;
}

long CKFXGLView::onUpdObject(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg = m_fxuiManipulate == OBJECT ? ID_CHECK : ID_UNCHECK;
    sender->handle(this, MKUINT(ID_SHOW, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(ID_ENABLE, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(msg, SEL_COMMAND), NULL);
    FXButton* fxb = (FXButton*)sender;
    if(msg == ID_CHECK){
	fxb->setState(STATE_DOWN);
    }else{
	fxb->setState(STATE_UP);
    }

    return 1;
}

long CKFXGLView::onCmdPoints(FXObject*, FXSelector, void*)
{
    m_fxuiRenderState = POINTS;
    callonPaint();

    return 1;
}

long CKFXGLView::onUpdPoints(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg = m_fxuiRenderState == POINTS ? ID_CHECK : ID_UNCHECK;
    sender->handle(this, MKUINT(ID_SHOW, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(ID_ENABLE, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(msg, SEL_COMMAND), NULL);

    return 1;
}

long CKFXGLView::onCmdWireframe(FXObject*, FXSelector, void*)
{
    m_fxuiRenderState = WIREFRAME;
    callonPaint();
    return 1;
}

long CKFXGLView::onUpdWireframe(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg = m_fxuiRenderState == WIREFRAME ? ID_CHECK : ID_UNCHECK;
    sender->handle(this, MKUINT(ID_SHOW, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(ID_ENABLE, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(msg, SEL_COMMAND), NULL);

    return 1;
}

long CKFXGLView::onCmdSurface(FXObject*, FXSelector, void*)
{
    m_fxuiRenderState = SURFACE;
    callonPaint();
    return 1;
}

long CKFXGLView::onUpdSurface(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg = m_fxuiRenderState == SURFACE ? ID_CHECK : ID_UNCHECK;
    sender->handle(this, MKUINT(ID_SHOW, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(ID_ENABLE, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(msg, SEL_COMMAND), NULL);

    return 1;
}

long CKFXGLView::onCmdColor(FXObject*, FXSelector, void*)
{
    m_fxuiColor = m_fxuiColor == COLOR ? NOCOLOR : COLOR;
    callonPaint();

    return 1;
}


long CKFXGLView::onUpdColor(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg = m_fxuiColor == COLOR ? ID_CHECK : ID_UNCHECK;
    sender->handle(this, MKUINT(ID_SHOW, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(ID_ENABLE, SEL_COMMAND), NULL);
    sender->handle(this, MKUINT(msg, SEL_COMMAND), NULL);
    FXButton* fxb = (FXButton*)sender;
    if(msg == ID_CHECK){
	fxb->setState(STATE_DOWN);
    }else{
	fxb->setState(STATE_UP);
    }

    return 1;
}

long CKFXGLView::onLeftBtnPress(FXObject*, FXSelector, void* ptr)
{
    m_fxuiRenderStateBack = m_fxuiRenderState;
    m_fxuiRenderState = POINTS;
    callonPaint();

    return 1;
}

long CKFXGLView::onLeftBtnRelease(FXObject*, FXSelector, void* ptr)
{
    m_fxuiRenderState = m_fxuiRenderStateBack;
    callonPaint();

    return 1;
}

long CKFXGLView::onRightBtnPress(FXObject*, FXSelector, void* ptr)
{
    m_fxuiRenderStateBack = m_fxuiRenderState;
    m_fxuiRenderState = POINTS;
    callonPaint();

    return 1;
}

long CKFXGLView::onRightBtnRelease(FXObject*, FXSelector, void* ptr)
{
    m_fxuiRenderState = m_fxuiRenderStateBack;
    callonPaint();

    return 1;
}

long CKFXGLView::onCmdDumpImage(FXObject*, FXSelector, void* ptr)
{
    std::string strFName("dump.ppm");
    dumpImage(strFName);

    return 1;
}




void CKFXGLView::rotateView(NPoint& npLast, NPoint& npCurr)
{
    float f0x, f0y, f1x, f1y;
    f0x = 2.0*npLast.nx/m_nVWidth-1.0;
    f0y = 1.0-2.0*npLast.ny/m_nVHeight;
    f1x = 2.0*npCurr.nx/m_nVWidth-1.0;
    f1y = 1.0-2.0*npCurr.ny/m_nVHeight;

    float pfspin[4];
	trackball(pfspin, f0x, f0y, f1x, f1y);
	add_quats(pfspin, m_qObj, m_qObj);
    
    float dum[4][4];

    build_rotmatrix(dum, m_qObj);
    for(int i = 0; i < 4; i++){
	for(int j = 0; j < 4; j++){
	    m_pfRotMat[i*4+j] = dum[i][j];
	}
    }

    return;
}

void CKFXGLView::transView(NPoint& npLast, NPoint& npCurr)
{
    /* translate X and Y of the view matrix */
    m_pfTransMat[12] += TRANS_VIEW_SCALE*(npCurr.nx-npLast.nx)/(float)m_nVHeight;
    m_pfTransMat[13] += TRANS_VIEW_SCALE*-(npCurr.ny-npLast.ny)/(float)m_nVHeight;

    return;
}

void CKFXGLView::zoomView(NPoint& npLast, NPoint& npCurr)
{
    /* scale view matrix */
    m_fScale += (npCurr.ny-npLast.ny)/(float)m_nVHeight;

    return;
}

void CKFXGLView::rotateObject(const int nID, NPoint& npLast, NPoint& npCurr)
{
    float f0x, f0y, f1x, f1y;
    f0x = 2.0*npLast.nx/m_nVWidth-1.0;
    f0y = 1.0-2.0*npLast.ny/m_nVHeight;
    f1x = 2.0*npCurr.nx/m_nVWidth-1.0;
    f1y = 1.0-2.0*npCurr.ny/m_nVHeight;

    float pfspin[4];
    trackball(pfspin, f0x, f0y, f1x, f1y);
   
    float dum[4][4];
    build_rotmatrix(dum, pfspin);
	
    glmat_mult_mat4x4(m_vmatModel[nID], dum);
    return;
}

void CKFXGLView::transObject(const int nID, NPoint& npLast, NPoint& npCurr, bool bZ)
{
    float pfMat[16];
    setIdentityMat(pfMat);
    if(bZ){ // translate on the Z axis
	pfMat[14] += TRANS_OBJECT_SCALE*-(npLast.ny-npCurr.ny)/(float)m_nVHeight;
    }else{
	pfMat[12] += TRANS_OBJECT_SCALE*-(npLast.nx-npCurr.nx)/(float)m_nVWidth;
	pfMat[13] += TRANS_OBJECT_SCALE*(npLast.ny-npCurr.ny)/(float)m_nVHeight;
    }

    glmat_mult_mat(m_vmatModel[nID], pfMat);

    return;
}


// Misc. utils
void CKFXGLView::clearVPort()
{
    glClearDepth(1.0);
    glClearColor(0.0, 0.0, 0.0, 1.0);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
}

void CKFXGLView::printEye() const
{
    std::cout << "Eye: " << m_fvEye[0] << ", " << m_fvEye[1] << ", " << m_fvEye[2] << std::endl;
    std::cout << "Center: " << m_fvCenter[0] << ", " << m_fvCenter[1] << ", " << m_fvCenter[2] << std::endl;
    std::cout << "Up: " << m_fvUp[0] << ", " << m_fvUp[1] << ", " << m_fvUp[2] << std::endl;
}


void CKFXGLView::printViewport() const
{
    GLfloat pfvp[4];
    glGetFloatv(GL_VIEWPORT, pfvp);
    std::cout << "Viewport: " << pfvp[0] << ", " << pfvp[1] << ", " << pfvp[2] << ", " << pfvp[3] << std::endl;
}

void CKFXGLView::printProjMat() const
{
    GLfloat pfpm[16];
    glGetFloatv(GL_PROJECTION_MATRIX, pfpm);
    std::cout << "Projection Matrix: " << std::endl;
    std::cout.setf(std::ios::fixed);
    for(int i = 0; i < 4; i++){
	for(int j = 0; j < 4; j++){
	    std::cout << "\t";
	    std::cout << pfpm[i*4+j];
	}
	std::cout << std::endl;
    }
}

void CKFXGLView::printMViewMat() const
{
    GLfloat pfmv[16];
    glGetFloatv(GL_MODELVIEW_MATRIX, pfmv);
    std::cout << "ModelView Matrix: " << std::endl;
    std::cout.setf(std::ios::fixed);
    for(int i = 0; i < 4; i++){
	for(int j = 0; j < 4; j++){
	    std::cout << "\t";
	    std::cout << pfmv[i*4+j];
	}
	std::cout << std::endl;
    }
}

void CKFXGLView::renderSRObject(const SROBJ* pobj, const int nID, const int nFlag, const bool bColor) const
{
    unsigned long ulID;
    float* basec;
    basec = basecolors[nID%NBC];
    if(nFlag == POINTS){
	glColor3f(basec[0], basec[1], basec[2]);
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FLOAT, 0, (const void*)pobj->pfC);
	glDrawArrays(GL_POINTS, 0, pobj->ulNVerts);
    }else if(nFlag == WIREFRAME){
	float pc[4];
	for(unsigned long ul = 0; ul < pobj->ulNFaces; ul++){
	    glPolygonMode(GL_FRONT, GL_LINE);
	    glBegin(GL_LINES);
	    ulID = pobj->pulF[ul*3];
	    if(!bColor) glColor3f(basec[0], basec[1], basec[2]);
	    else{
		Utils::Intensity2RGB(pc, pobj->pfI[ulID], basec);
		glColor4f(pc[0], pc[1], pc[2], pc[3]);
	    }
	    glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	    ulID = pobj->pulF[ul*3+1];
	    if(!bColor) glColor3f(basec[0], basec[1], basec[2]);
	    else{
		Utils::Intensity2RGB(pc, pobj->pfI[ulID], basec);
		glColor4f(pc[0], pc[1], pc[2], pc[3]);
	    }
	    glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	    ulID = pobj->pulF[ul*3+2];
	    if(!bColor) glColor3f(basec[0], basec[1], basec[2]);
	    else{
		Utils::Intensity2RGB(pc, pobj->pfI[ulID], basec);
		glColor4f(pc[0], pc[1], pc[2], pc[3]);
	    }
	    glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	    glEnd();
	}
    }else if(nFlag == SURFACE){
	float pc[4];
	for(unsigned long ul = 0; ul < pobj->ulNFaces; ul++){
	    glPolygonMode(GL_FRONT, GL_FILL);	    
	    glBegin(GL_TRIANGLES);
	    ulID = pobj->pulF[ul*3];
	    if(!bColor) glColor3f(basec[0], basec[1], basec[2]);
	    else{
		Utils::Intensity2RGB(pc, pobj->pfI[ulID], basec);
		glColor4f(pc[0], pc[1], pc[2], pc[3]);
	    }
	    glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	    ulID = pobj->pulF[ul*3+1];
	    if(!bColor) glColor3f(basec[0], basec[1], basec[2]);
	    else{
		Utils::Intensity2RGB(pc, pobj->pfI[ulID], basec);
		glColor4f(pc[0], pc[1], pc[2], pc[3]);
	    }
	    glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	    ulID = pobj->pulF[ul*3+2];
	    if(!bColor) glColor3f(basec[0], basec[1], basec[2]);
	    else{
		Utils::Intensity2RGB(pc, pobj->pfI[ulID], basec);
		glColor4f(pc[0], pc[1], pc[2], pc[3]);
	    }
	    glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	    glEnd();
	}
    }else{
    }

}    

void CKFXGLView::makeGrid() const
{
    glBegin(GL_LINES);
    glColor3f(0.8, 0.0, 0.2);
    glVertex3d(0, 0, 0);
    glVertex3d(10, 0, 0);
    glColor3f(0.0, 0.8, 0.2);
    glVertex3d(0, 0, 0);
    glVertex3d(0, 10, 0);
    glColor3f(0.2, 0.0, 0.8);
    glVertex3d(0, 0, 0);
    glVertex3d(0, 0, 10);
    glEnd();
}


int CKFXGLView::computeSceneList(const float fAngle, const float fScale, 
				 const float fPercent, bool** pbScnList)
{

    int nCurrentRState = m_fxuiRenderState;
    m_fxuiRenderState = SURFACE;
    callonPaint();
    
    float fBase, fCnt;

    for(int nMID = 0; nMID < m_vObjs.size(); nMID++){
	fBase = (float)dumpCheckImage(nMID, nMID, fAngle, fScale);
	std::cout << "nMID: " << nMID << " -> " << (int)fBase << ", ";
	for(int nSID = 0; nSID < m_vObjs.size(); nSID++){
	    std::cout << nSID << " -> ";
	    if(nSID == nMID){
		pbScnList[nMID][nSID] = false; // since it's itself
		std::cout << "--" << " ";
	    }else{
		fCnt = (float)dumpCheckImage(nMID, nSID, fAngle, fScale);
		std::cout << (int)fCnt << " ";
		if(fCnt > fBase*fPercent){
		    pbScnList[nMID][nSID] = true;
		}else{
		    pbScnList[nMID][nSID] = false;
		}
	    }
	    
	}
	std::cout << std::endl;
    }

    m_fxuiRenderState = nCurrentRState;
    callonPaint();

    
    return 0;
}
	
/*
 * The view point must be computed inside this function
 */
int CKFXGLView::dumpCheckImage(const int nMID, const int nSID, 
				const float fAngle, const float fScale)
{
    // first of all, compute the view point 
    // get the current centroid and normal first
    gsl_vector* vAvgN = gsl_vector_alloc(4);
    gsl_vector* vCntrd = gsl_vector_alloc(4);
    m_vObjs[nMID]->getAvgNinGSL(vAvgN); // already rotated
    m_vObjs[nMID]->getCntrdinGSL(vCntrd); // already transformed
    double dLength;
    dLength = m_vObjs[nMID]->getMaxLength()/tan((fAngle*DEG2RAD)/2.0);

    gsl_vector_scale(vAvgN, dLength*fScale);
    gsl_vector* vVP = gsl_vector_alloc(4);
    gsl_vector_memcpy(vVP, vCntrd);
    gsl_vector_add(vVP, vAvgN);

    makeCurrent();
    glDrawBuffer(GL_BACK);
    renderOneRImage(nSID, vVP, vCntrd, fAngle, m_nVWidth, m_nVHeight);
    GLubyte* pPixs;
    pPixs = new GLubyte[m_nVWidth*m_nVHeight*3];
    glReadBuffer(GL_BACK);
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    glReadPixels(0, 0, m_nVWidth, m_nVHeight, GL_RGB, GL_UNSIGNED_BYTE, pPixs);

    swapBuffers();
    makeNonCurrent();

    //DEBUG: Utils::dumpPBytesAsPPM((unsigned char*)pPixs, m_nVWidth, m_nVHeight, "dump.ppm");
    int nNonBlack(0);
    nNonBlack = Utils::countNonBlackPixs((unsigned char*)pPixs, m_nVWidth, m_nVHeight);

    gsl_vector_free(vAvgN);
    gsl_vector_free(vCntrd);
    gsl_vector_free(vVP);
    delete [] pPixs;

    return nNonBlack;
}

void CKFXGLView::renderOneRImage(const int nID, gsl_vector* vVP, gsl_vector* vLA, 
				 const float fAngle, const int nWidth, const int nHeight)
{

    glEnable(GL_CULL_FACE);
    glFrontFace(GL_CW);
    glCullFace(GL_BACK);    
    glViewport(0, 0, nWidth, nHeight);
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    gluPerspective(fAngle, (GLfloat)nWidth/(GLfloat)nHeight, m_fNear, m_fFar);
    gluLookAt(gsl_vector_get(vVP, 0), gsl_vector_get(vVP, 1), gsl_vector_get(vVP, 2),
	      gsl_vector_get(vLA, 0), gsl_vector_get(vLA, 1), gsl_vector_get(vLA, 2),
	      0.0, 1.0, 0.0);

    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glMultMatrixf(m_vmatModel[nID]);
    glEnableClientState(GL_VERTEX_ARRAY);
    glCullFace(GL_BACK);
    glFrontFace(GL_CW);

    SROBJ* pobj = m_vObjs[nID];
    unsigned long ulID;
    glPolygonMode(GL_FRONT, GL_FILL);	    
    for(unsigned long ul = 0; ul < pobj->ulNFaces; ul++){
	glBegin(GL_TRIANGLES);
	glColor3f(basecolors[nID%NBC][0], basecolors[nID%NBC][1], basecolors[nID%NBC][2]);
	ulID = pobj->pulF[ul*3];
	glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	ulID = pobj->pulF[ul*3+1];
	glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	ulID = pobj->pulF[ul*3+2];
	glVertex3f(pobj->pfC[ulID*3], pobj->pfC[ulID*3+1], pobj->pfC[ulID*3+2]);
	glEnd();
    }


    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();

    glFlush();


}

int CKFXGLView::dumpImage(const std::string& strFName)
{
    makeCurrent();
    drawScene();
    GLubyte* pPixs;
    pPixs = new GLubyte[m_nVWidth*m_nVHeight*3];
    glReadBuffer(GL_BACK);
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    glReadPixels(0, 0, m_nVWidth, m_nVHeight, GL_RGB, GL_UNSIGNED_BYTE, pPixs);

    swapBuffers();
    makeNonCurrent();

    Utils::dumpPBytesAsPPM((unsigned char*)pPixs, m_nVWidth, m_nVHeight, strFName.c_str());

    delete [] pPixs;
    
    return 0;
}
