/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  KFXGLView.h
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#ifndef _KFXGLVIEW_H_
#define _KFXGLVIEW_H_

#include <fox/fx.h>
#include <fox/fx3d.h>
#ifndef FXGLCANVAS_H
#include <fox/FXGLCanvas.h>
#endif

#include "PSReg.h"

class FXAPI CKFXGLView : public FXGLCanvas
{
    FXDECLARE(CKFXGLView)
    friend class KGLObject;

    
public:
    CKFXGLView(); 
    virtual ~CKFXGLView();

    CKFXGLView(FXComposite* p, FXGLVisual* vis, FXObject* tgt=NULL, \
	       FXSelector sel=0, FXuint opts=0, \
	       FXint x=0, FXint y=0, FXint w=0, FXint h=0);

    CKFXGLView(FXComposite* p,FXGLVisual* vis, FXGLViewer* sharegroup, \
	       FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0, \
	       FXint x=0, FXint y=0, FXint w=0, FXint h=0);
    
    inline void addObject(SROBJ* psrobj, FXTreeItem* pfxti){
	m_vObjs.push_back(psrobj);
	GLfloat* pmat = new GLfloat[16];
	m_vTItems.push_back(pfxti);
	setIdentityMat(pmat);
	m_vmatModel.push_back(pmat);
	callonPaint();
    }
    inline void cleanObject(){
	m_vObjs.clear();
	std::vector<GLfloat*>::iterator itr;
	for(itr = m_vmatModel.begin(); itr != m_vmatModel.end(); itr++){
	    delete [] *itr;
	}
	m_vmatModel.clear();
	m_vTItems.clear();
    }
    inline void getSelected(bool* pbSelected){
	std::vector<FXTreeItem*>::iterator itr;
	int ncnt(0);
	for(itr = m_vTItems.begin(); itr != m_vTItems.end(); itr++){
	    if((*itr)->isSelected()) pbSelected[ncnt] = true;
	    else pbSelected[ncnt] = false;
	    ncnt++;
	}
    }
    inline void initializeModelMats(){
	std::vector<GLfloat*>::iterator itr;
	for(itr = m_vmatModel.begin(); itr != m_vmatModel.end(); itr++){
	    setIdentityMat((*itr));
	}
    }
    inline const float* getModelMat(const int nID) const {
	return m_vmatModel[nID];
    }
    inline std::vector<GLfloat*>* getModelMats(){
	return &m_vmatModel;
    }
	
    int computeSceneList(const float fAngle, const float fScale, 
			 const float fPercent, bool** pbScnList);
    int dumpImage(const std::string& strFName);


    long callonPaint(){ return onPaint(NULL, 0, NULL); }

private:
    enum {
	CAMERA,
	OBJECT,
	FREEZE
    };

    enum {
	POINTS,
	WIREFRAME,
	SURFACE
    };

    enum {
	COLOR,
	NOCOLOR
    };
    
    int m_nVWidth, m_nVHeight; // Viewport width and height
    FXVec3f m_fvEye, m_fvCenter, m_fvUp; // View point
    float m_fAngle, m_fNear, m_fFar; // stuff for gluPerspective
    bool m_bProj; // is the projection matrix set?
    float m_pfRotMat[16], m_pfTransMat[16];
    float m_fScale;
    FXuint m_fxuiManipulate, m_fxuiRenderState, m_fxuiColor;
    FXuint m_fxuiRenderStateBack;
    
    float* m_qObj; // object quaternion
    
    std::vector<SROBJ*> m_vObjs;
    std::vector<GLfloat*> m_vmatModel;
    std::vector<FXTreeItem*> m_vTItems;
    
    
    void initialize();
    void create();
    void glsetup();
    void drawScene();
    void renderSRObject(const SROBJ* pobj, const int nID, const int nFlag, const bool bColor = false) const;
    void makeGrid() const;
    
    void rotateView(NPoint& npLast, NPoint& npCurr);
    void transView(NPoint& npLast, NPoint& npCurr);
    void zoomView(NPoint& npLast, NPoint& npCurr);
    void rotateObject(const int nID, NPoint& npLast, NPoint& npCurr);
    void transObject(const int nID, NPoint& npLast, NPoint& npCurr, bool bZ);

    int dumpCheckImage(const int nMID, const int nSID, const float fAngle, const float fScale);
    void renderOneRImage(const int nID, gsl_vector* vVP, gsl_vector* vLA, 
			 const float fAngle, const int nWidth, const int nHeight);
    /// Utils
    void clearVPort();
    void printEye() const;
    void printViewport() const;
    void printProjMat() const;
    void printMViewMat() const;
    inline void printMat(float pMat[16]) const{
	std::cout.setf(std::ios::fixed);
	for(int i = 0; i < 4; i++){
	    for(int j = 0; j < 4; j++){
		std::cout << "\t" << pMat[i*4+j];
	    }
	    std::cout << "\n";
	}
    }
		
    inline void setIdentityMat(GLfloat pMat[16]){
	for(int i = 0; i < 4; i++){
	    for(int j = 0; j < 4; j++){
		if(i == j) pMat[i*4+j] = 1.0;
		else pMat[i*4+j] = 0.0;
	    }
	}
    }

    inline void glmat_mult_mat4x4(GLfloat* pglmat, float pmat4x4[4][4]);
    inline void glmat_mult_mat(GLfloat* pglmat, float* pmat);


public:
    // Messages
    enum{
	ID_ABOUT = FXGLCanvas::ID_LAST,
	    ID_CAMERA,
	    ID_OBJECT,
	    ID_POINTS,
	    ID_WIREFRAME,
	    ID_SURFACE,
	    ID_DUMP,
	    ID_COLOR,
	    ID_LAST
	    };


    /// Events
    // drawing
    long onPaint(FXObject*, FXSelector, void*);
    // mouse bindings
    long onMouseMove(FXObject*, FXSelector, void* ptr);

    long onCmdCamera(FXObject*, FXSelector, void*);
    long onUpdCamera(FXObject* sender, FXSelector, void*);
    long onCmdObject(FXObject*, FXSelector, void*);
    long onUpdObject(FXObject* sender, FXSelector, void*);
    long onCmdPoints(FXObject*, FXSelector, void*);
    long onUpdPoints(FXObject* sender, FXSelector, void*);
    long onCmdWireframe(FXObject*, FXSelector, void*);
    long onUpdWireframe(FXObject* sender, FXSelector, void*);
    long onCmdSurface(FXObject*, FXSelector, void*);
    long onUpdSurface(FXObject* sender, FXSelector, void*);
    long onCmdDumpImage(FXObject* sender, FXSelector, void*);
    long onCmdColor(FXObject*, FXSelector, void*);
    long onUpdColor(FXObject* sender, FXSelector, void*);

    long onLeftBtnPress(FXObject*, FXSelector, void*);
    long onLeftBtnRelease(FXObject*, FXSelector, void*);
    long onRightBtnPress(FXObject*, FXSelector, void*);
    long onRightBtnRelease(FXObject*, FXSelector, void*);
    


};

inline void CKFXGLView::glmat_mult_mat4x4(GLfloat* pglmat, float pmat4x4[4][4])
{
    float pfR[16];
    int i, j, k;
    for(i = 0; i <  16; i++) pfR[i] = 0.0;
    for(i = 0; i < 4; i++){
	for(j = 0; j < 4; j++){
	    for(k = 0; k < 4; k++){
		pfR[i*4+j] += pglmat[i*4+k]*pmat4x4[k][j];
	    }
	}
    }
    for(i = 0; i < 16; i++) pglmat[i] = pfR[i];
}


inline void CKFXGLView::glmat_mult_mat(GLfloat* pglmat, float* pmat)
{
    float pfR[16];
    int i, j, k;
    for(i = 0; i <  16; i++) pfR[i] = 0.0;
    for(i = 0; i < 4; i++){
	for(j = 0; j < 4; j++){
	    for(k = 0; k < 4; k++){
		pfR[i*4+j] += pglmat[i*4+k]*pmat[k*4+j];
	    }
	}
    }
    for(i = 0; i < 16; i++) pglmat[i] = pfR[i];
}

#endif // _KFXGLVIEW_H_
