#ifndef _KONSTRUCTS_H_
#define _KONSTRUCTS_H_

// left for compatibility in C3DFile
typedef struct _CPoint{
    int cx, cy;
} CPoint;


typedef struct _CLBPNT{
    double dX, dY, dZ;
    double dx, dy;
} CLBPNT;

typedef struct _LRGN{
    int nID;
    CPoint* pPnts; // 2D coordinates of the points in the region
    int nNPnts; // Number of points in the region
    float fCx, fCy; // Centroid
} LRGN;

typedef LRGN* PLRGN;

typedef struct _NPoint{
    int nx, ny;
    _NPoint& operator=(const _NPoint& ref){
	if(this == &ref) return *this;
	nx = ref.nx;
	ny = ref.ny;
	return *this;
    }	
} NPoint;

typedef struct _FPoint{
    float fx, fy;
    _FPoint& operator=(const _FPoint& ref){
	if(this == &ref) return *this;
	fx = ref.fx;
	fy = ref.fy;
	return *this;
    }
} FPoint;

typedef struct _DPoint{
    double dx, dy;
    
    _DPoint& operator=(const _DPoint& ref){
	if(this == &ref) return *this;
	dx = ref.dx;
	dy = ref.dy;
	return *this;
    }
} DPoint;

typedef struct _F3DPoint{
    float fx, fy, fz;

    _F3DPoint& operator=(const _F3DPoint& ref){
	if(this == &ref) return *this;
	fx = ref.fx;
	fy = ref.fy;
	fz = ref.fz;
	return *this;
    }
    float operator*(const _F3DPoint& ref){
	return (this->fx*ref.fx+this->fy*ref.fy+this->fz*ref.fz);
    }
} F3DPoint;

typedef struct _D3DPoint{
    double dx, dy, dz;
    _D3DPoint& operator=(const _D3DPoint& ref){
	if(this == &ref) return *this;
	dx = ref.dx;
	dy = ref.dy;
	dz = ref.dz;
	return *this;
    }
    double operator*(const _D3DPoint& ref){
	return (this->dx*ref.dx+this->dy*ref.dy+this->dz*ref.dz);
    }
} D3DPoint;

typedef struct _TRNGL{
    unsigned long nF, nS, nT;
    F3DPoint normal;
    _TRNGL& operator=(const _TRNGL& ref){
	if(this == &ref) return *this;
	normal = ref.normal;
	nF = ref.nF;
	nS = ref.nS;
	nT = ref.nT;
	
	return *this;
    }
} TRNGL;

typedef struct _VRTX{
    F3DPoint pos; // 3D coordinate
    F3DPoint normal;
//    DPoint tcrd; // texture coordinate
    bool boundary;
//    unsigned char R, G, B, A;
    float Y;
    
    _VRTX& operator=(const _VRTX& ref){
	if(this == &ref) return *this;
	
	pos = ref.pos;
	normal = ref.normal;
//	tcrd = ref.tcrd;
	boundary = ref.boundary;
//	R = ref.R;
//	G = ref.G;
//	B = ref.B;
//	A = ref.A;
	Y = ref.Y;
	
	return *this;
	}
} VRTX;
	

#endif // _KONSTRUCTS_H_
