/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  PSRegWin.cpp
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#include "KDefs.h"
#include <fox/fx.h>
#include <fox/fx3d.h>
#include <iostream>
#include "3DFile.h"
#include "PSRegWin.h"
#include "icons.h"
#include "RegExecDlg.h"

#define FILE_SCALE 100.0

FXDEFMAP(CPSRegWin) CPSRegWinMap[] = {
    FXMAPFUNC(SEL_COMMAND, CPSRegWin::ID_ABOUT, CPSRegWin::onCmdAbout),
    FXMAPFUNC(SEL_COMMAND, CPSRegWin::ID_OPEN, CPSRegWin::onCmdOpen),
    FXMAPFUNC(SEL_COMMAND, CPSRegWin::ID_SAVEAS, CPSRegWin::onCmdSaveAs),
    FXMAPFUNC(SEL_COMMAND, CPSRegWin::ID_REGEXEC, CPSRegWin::onCmdRegExec),
};
		
FXIMPLEMENT(CPSRegWin, FXMainWindow, CPSRegWinMap, ARRAYNUMBER(CPSRegWinMap))

CPSRegWin::CPSRegWin(FXApp* fxApp) 
    : FXMainWindow(fxApp, "Parallel Simultaneous Registration", NULL, NULL, DECOR_ALL, 0, 0, 800, 600)//1024, 768)
{
    m_nCnt = 0;
    m_nNObjs = 0;
    m_nNSteps = 5;
    m_nNRKSteps = 5;
    m_nNLMSteps = 5;
    m_fMESigma = 0.1;
    m_fNThresh = -1.0;
    m_fTDSearch = 0.01;
    m_fCGSearch = 0.01;
    m_nBCheck = 1;
    m_nIEval = 0;
    m_nMultiNP = 10;
    m_nPair = 0;
    m_fBack = 0.3;
    m_nDump = 1;

    // Menubar---------------
    m_menub = new FXMenuBar(this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
    // ----------------------
    new FXHorizontalSeparator(this, LAYOUT_SIDE_TOP|SEPARATOR_GROOVE|LAYOUT_FILL_X);
    // Toolbar---------------
    m_toolb = new FXToolBar(this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X, \
			    0, 0, 0, 0, 4, 4, 4, 4, 0, 0);

    // ----------------------
    new FXHorizontalSeparator(this, LAYOUT_SIDE_TOP|SEPARATOR_GROOVE|LAYOUT_FILL_X);

    FXHorizontalFrame* fxhfrmMain;
    fxhfrmMain = new FXHorizontalFrame(this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y, \
				       0, 0, 0, 0, 10, 10, 10, 10);


    FXSplitter* fxsplit;
    fxsplit = new FXSplitter(fxhfrmMain,
			     LAYOUT_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_REVERSED|SPLITTER_TRACKING|SPLITTER_HORIZONTAL);

    
    // GLView----------------
    FXComposite* fxcmpGL;
    fxcmpGL = new FXVerticalFrame(fxsplit, 
				  FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH,
				  0, 0, 0, 0, 0, 0, 0, 0);

    m_glvisual = new FXGLVisual(getApp(), VISUAL_DOUBLE_BUFFER);
    m_glview = new CKFXGLView(fxcmpGL, m_glvisual, this, ID_GLVIEW, LAYOUT_FILL_X|LAYOUT_FILL_Y,
			      0, 0, 0, 0);

    // Treelist--------------
    FXVerticalFrame* fxvfrmTree;
    fxvfrmTree = new FXVerticalFrame(fxsplit, 
				     FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH,
				     0, 0, 0, 0, 0, 0, 0, 0);

    //m_treel = new FXTreeList(fxvfrmTree, 0, NULL, 0, 
    m_treel = new FXTreeList(fxvfrmTree, NULL, 0, 
			     FRAME_SUNKEN|FRAME_THICK|LAYOUT_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_SIDE_LEFT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES|TREELIST_EXTENDEDSELECT);


    // prepare icons
    FXIcon *fxicKON = new FXGIFIcon(getApp(), icthreesmilesmall, 0, IMAGE_ALPHAGUESS);
    FXIcon *fxicOpen = new FXGIFIcon(getApp(), icfileopen);
    FXIcon *fxicSave = new FXGIFIcon(getApp(), icfilesave);
    FXIcon *fxicSaveAs = new FXGIFIcon(getApp(), icfilesaveas, 0, IMAGE_ALPHAGUESS);
    FXIcon *fxicCamera = new FXGIFIcon(getApp(), iccamsmall, 0, IMAGE_ALPHAGUESS);
    FXIcon *fxicWorld = new FXGIFIcon(getApp(), icobject, 0, IMAGE_ALPHAGUESS);
    FXIcon *fxicPoint = new FXGIFIcon(getApp(), icpoint, 0, IMAGE_ALPHAGUESS);
    FXIcon *fxicWire = new FXGIFIcon(getApp(), icwire, 0, IMAGE_ALPHAGUESS);
    FXIcon *fxicSurface = new FXGIFIcon(getApp(), icsurface, 0, IMAGE_ALPHAGUESS);
    FXIcon *fxicGlass = new FXGIFIcon(getApp(), icglass, 0, IMAGE_ALPHAGUESS);
    FXIcon *fxicPenguin = new FXGIFIcon(getApp(), icpenguin);

    // Set up menus
    m_filem = new FXMenuPane(this);
    new FXMenuTitle(m_menub, "&File", NULL, m_filem);
    new FXMenuCommand(m_filem, "&Open...\tCtl-O\tOpen 3D file.", fxicOpen, this, ID_OPEN);
    new FXMenuCommand(m_filem, "&Save\tCtl-S\tSave 3D files.", fxicSave);
    new FXMenuCommand(m_filem, "Save &As...\t\tSave 3D files to other files.", fxicSaveAs, this, ID_SAVEAS);
    new FXMenuCommand(m_filem, "&Quit\tCtl-Q\tQuit the application.", NULL, getApp(), FXApp::ID_QUIT);

    m_viewm = new FXMenuPane(this);
    new FXMenuTitle(m_menub, "&View", NULL, m_viewm);
    new FXMenuCommand(m_viewm, "&Dump screen\t\tDump screen.", NULL, m_glview, CKFXGLView::ID_DUMP);

    m_regm = new FXMenuPane(this);
    new FXMenuTitle(m_menub, "&Register", NULL, m_regm);
    new FXMenuCommand(m_regm, "&Excute...\t\tExcute registration.", NULL, this, ID_REGEXEC);

    // in Toolbar: File manipulation
    new FXFrame(m_toolb, LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 4, 20);
    new FXButton(m_toolb, "\tOpen\tOpen 3D file.", fxicOpen, this, ID_OPEN, FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // in Toolbar: View or object
    new FXFrame(m_toolb, LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 8, 20);
    new FXButton(m_toolb, "\tView\tManipulate view.", fxicCamera, m_glview, CKFXGLView::ID_CAMERA, FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    new FXButton(m_toolb, "\tObject\tManipulate object.", fxicWorld, m_glview, CKFXGLView::ID_OBJECT, FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // in Toolbar: RenderState
    new FXFrame(m_toolb, LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 8, 20);
    new FXButton(m_toolb, "\tPoints\tRender in points mode.", fxicPoint, m_glview, CKFXGLView::ID_POINTS, FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    new FXButton(m_toolb, "\tWireframe\tRender in wireframe mode.", fxicWire, m_glview, CKFXGLView::ID_WIREFRAME, FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    new FXButton(m_toolb, "\tSurface\tRender in surface mode.", fxicSurface, m_glview, CKFXGLView::ID_SURFACE, FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // in Toolbar: Color
    new FXFrame(m_toolb, LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 8, 20);
    new FXButton(m_toolb, "\tColor\tRender with reflectance.", fxicGlass, m_glview, CKFXGLView::ID_COLOR, FRAME_THICK|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // Help menu
    m_helpm = new FXMenuPane(this);
    new FXMenuTitle(m_menub, "&Help", NULL, m_helpm, LAYOUT_RIGHT);
    new FXMenuCommand(m_helpm, "&About Fox...\t\tDisplay FOX about panel.", NULL, this, ID_ABOUT, 0);

    // Prepare the registration class
    m_ppsr = new CPSReg;
}


CPSRegWin::~CPSRegWin()
{
    delete m_toolb;
    delete m_menub;
    delete m_filem;
    delete m_glvisual;
    delete m_ppsr;
}


void CPSRegWin::create()
{
    FXMainWindow::create();
    show(PLACEMENT_SCREEN);
}

long CPSRegWin::onCmdAbout(FXObject*, FXSelector, void*)
{
    FXMessageBox::information(this, MBOX_OK, "About PSReg", 
			      "Parallel Simultaneous Registration of Multiple Range Images\n\nCopyright (C) 2002 Ko Nishino\nkon@computer.org\nComputer Vision Laboratory\nInstitute of Industrial Science\nThe University of Tokyo");
    return 1;
}


/* Open 3D Files
*/
long CPSRegWin::onCmdOpen(FXObject*, FXSelector, void*)
{
/*
    const FXchar *fxchPatts[] = {
//	"Ply Files", "*.ply",
//	"Pts Files", "*.pts",
//	"VRML Files", "*.wrl",
	"All Files", "*",
	NULL};
    FXString fxsPatts = FXString(*fxchPatts);
*/
    const FXString fxsPatts = FXString("Ply Files (*.ply)\nAll Files (*)");
    FXFileDialog fxFileDlg(this, "Open 3D Files");
    fxFileDlg.setSelectMode(SELECTFILE_MULTIPLE);
    //fxFileDlg.setPatternList(fxsPatts);
    fxFileDlg.setPatternList(fxsPatts);
    if(fxFileDlg.execute()){
	FXString *strFNames = fxFileDlg.getFilenames();
	std::vector<std::string> vstrFNames;
	int nFNames = 0;
	while(strFNames[nFNames] != ""){
	    std::cout << strFNames[nFNames].text() << std::endl;
            vstrFNames.push_back(strFNames[nFNames].text());
	    nFNames++;
	}

/*
	std::string strFNames(fxFileDlg.getFilenames().text());
	std::vector<std::string> vstrFNames;
	getSprtFilenames(strFNames, vstrFNames);
*/
	std::vector<std::string>::iterator itr;
	std::cout << "Opening" << std::endl;
	
	for(itr = vstrFNames.begin(); itr != vstrFNames.end(); itr++){
	    C3DFile*  pc3dObj = new C3DFile;
	    pc3dObj->ReadFile(itr->c_str(), FILE_SCALE);
	    m_ppsr->addObject(pc3dObj);

	    FXTreeItem* pfxti = new FXTreeItem(pc3dObj->GetShortFName(), NULL, NULL);
	    m_treel->appendItem(0, pfxti);
	    m_glview->addObject(m_ppsr->getObj(m_nNObjs), pfxti);

	    m_nNObjs++;
	    delete pc3dObj;
	}
    }

    return 1;
}

/*
  getSprtFilenames
  Separate the file names from a concat list
 */
int CPSRegWin::getSprtFilenames(std::string& strFNames, std::vector<std::string>& vstrFNames)
{
    int nPos;
    std::cout << strFNames << std::endl;
    std::string strDQuot("\"");
    // get the path
    nPos = strFNames.find(strDQuot);
    std::string strPath;
    strPath = strFNames.substr(0, nPos);
    strFNames = strFNames.substr(nPos+1, strFNames.length());

    std::string strOneFName;
    while(1){ // get the file names
	nPos = strFNames.find(strDQuot);
	strOneFName = strFNames.substr(0, nPos);
	if(nPos != 1){
	    strOneFName = strPath + strOneFName;
	    vstrFNames.push_back(strOneFName);
	}
	if(nPos == strFNames.length()-1) break;
	strFNames = strFNames.substr(nPos+1, strFNames.length());
    }

     return 0;
}


long CPSRegWin::onCmdSaveAs(FXObject*, FXSelector, void*)
{
    int nNFiles = m_treel->getNumItems();

    FXTreeItem* fxTI = m_treel->getFirstItem();
    FXString fxsFName;
    std::string strFName;
    char pcCmnt[512];
    m_ppsr->updateMatrices(m_glview->getModelMats());
    m_glview->initializeModelMats();
    for(int i = 0; i < nNFiles; i++){
	if(fxTI->isSelected()){
	    fxsFName = fxTI->getText();
	    if(getSaveName(fxsFName, strFName)) continue;
	    std::cout << "saving " << fxsFName.text() << " as " << strFName.c_str() << std::endl;
	    C3DFile c3df;
	    m_ppsr->dump3DFile(i, &c3df); 
	    sprintf(pcCmnt, "originally %s", fxsFName.text());
	    c3df.SetComment(pcCmnt);
	    
	    c3df.WritePly(strFName.c_str(), (float)1.0/FILE_SCALE);
	}else{

	}
	fxTI = fxTI->getNext();
    }

    return 1;
}

int CPSRegWin::getSaveName(FXString& fxsFName, std::string& strFName)
{
    const FXchar *fxchPatts[] = {
	"Ply Files", "*.ply",
//	"Pts Files", "*.pts",
//	"VRML Files", "*.wrl",
	"All Files", "*",
	NULL};
    FXString fxsPatts = FXString(*fxchPatts);
    FXFileDialog fxFileDlg(this, "Save 3D Files");
    fxFileDlg.setFilename(fxsFName);
    fxFileDlg.setSelectMode(SELECTFILE_ANY);
    fxFileDlg.setPatternList(fxsPatts);
    if(fxFileDlg.execute()){
	strFName = fxFileDlg.getFilename().text();
	return 0;
    }else{
	return 1;
    }
}



	


long CPSRegWin::onCmdRegExec(FXObject*, FXSelector, void*)
{
    bool** pbScnList;
    FXint fxnNItems = m_treel->getNumItems();
    pbScnList = new (bool (*[fxnNItems]));
    for(int i = 0; i < m_treel->getNumItems(); i++){
	pbScnList[i] = new bool[m_treel->getNumItems()];
    }


    CRegExecDlg credlg(this, m_nNSteps, m_nNRKSteps, m_nNLMSteps, m_fMESigma, m_fNThresh, 
		       m_fTDSearch, m_fCGSearch, m_fBack, 
		       m_nBCheck, m_nIEval, m_nMultiNP, m_nPair, m_nDump);
    bool* pbModels;
    pbModels = new bool[m_treel->getNumItems()];
    m_glview->getSelected(pbModels);

    if(credlg.execute()){
	m_nNSteps = credlg.getNSteps();
	m_nNRKSteps = credlg.getNRKSteps();
	m_nNLMSteps = credlg.getNLMSteps();
	m_fMESigma = credlg.getMESigma();
	m_fNThresh = credlg.getNThresh();
	m_fTDSearch = credlg.getTDSearch();
	m_fCGSearch = credlg.getCGSearch();
	m_fBack = credlg.getBack();
	m_nBCheck = credlg.getBCheck();
	m_nIEval = credlg.getIEval();
	m_nMultiNP = credlg.getMultiNP();
	m_nPair = credlg.getPair();
	m_nDump = credlg.getDump();
	
	std::cout << "NSteps: " << m_nNSteps << " " 
		  << "NRKSteps: " << m_nNRKSteps << " "
		  << "NLMSteps: " << m_nNLMSteps << " "
		  << "MESigma: " << m_fMESigma << " "
		  << "NThresh: " << m_fNThresh << " "
		  << "TDSearch: " << m_fTDSearch << " "
		  << "CGSearch: " << m_fCGSearch << " "
		  << "Back: " << m_fBack << " " 
		  << "BCheck: " << m_nBCheck << " "
		  << "IEval: " << m_nIEval << " "
		  << "MultiNP: " << m_nMultiNP << " "
		  << "Pair: " << m_nPair << " "
		  << "Dump: " << m_nDump << std::endl;

	m_ppsr->updateMatrices(m_glview->getModelMats());
	m_glview->initializeModelMats();
	std::string strDump;
	char pcDump[512];
	if(m_nDump == 1 && m_nCnt == 0){
	    sprintf(pcDump, "%04d.ppm", m_nCnt);
	    strDump = pcDump;
	    m_glview->dumpImage(strDump);
	    m_nCnt++;
	}
	for(int i = 0; i < m_nNSteps; i++){
	    m_glview->computeSceneList(45.0, 0.8, m_fBack, pbScnList);
	    for(int i = 0; i < m_treel->getNumItems(); i++){
		std::cout << "ScnList[" << i << "]: ";
		for(int j = 0; j < m_treel->getNumItems(); j++){
		    std::cout << (pbScnList[i][j] == true ? "\ttrue" : "\tfalse") << " ";
		}
		std::cout << std::endl;
	    }

	    m_ppsr->runRegistration(m_nNRKSteps, m_nNLMSteps, m_fMESigma, m_fNThresh,
				    m_fTDSearch, m_fCGSearch,
				    (m_nBCheck == 0 ? false : true),
				    (m_nIEval == 0 ? false : true),
				    m_nMultiNP, 
				    (m_nPair == 0 ? false : true),
				    pbModels, pbScnList);
	    if(m_nDump == 1){
		sprintf(pcDump, "%04d.ppm", m_nCnt);
		strDump = pcDump;
		m_glview->dumpImage(strDump);
		m_nCnt++;
	    }else{
		m_glview->callonPaint();
	    }
	}


    }else{
	// nop
    }

    delete [] pbModels;
    for(int i = 0; i < m_treel->getNumItems(); i++){
	delete [] pbScnList[i];
    }
    delete [] pbScnList;

    return 1;
}

int main(int argc, char** argv)
{
    FXApp fxApp("PSReg", "Parallel Simultaneous Registration");
    fxApp.init(argc, argv);
    
    new CPSRegWin(&fxApp);
    fxApp.create();
    
    return fxApp.run();
}
   
  
