/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  PSRegWin.h              
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#ifndef _PSREGWIN_H_
#define _PSREGWIN_H_

#include <GL/gl.h>
#include <GL/glu.h>

#include <string>
#include <vector>
#include "PSReg.h"
#include "KFXGLView.h"

class CPSRegWin : public FXMainWindow
{    
    FXDECLARE(CPSRegWin)
public:
    CPSRegWin(FXApp* fxApp);
    virtual ~CPSRegWin();

    virtual void create();
    
    enum{
	ID_ABOUT = FXMainWindow::ID_LAST,
	    ID_OPEN,
	    ID_SAVEAS,
	    ID_VIEWCOLOR,
	    ID_REGEXEC,
	    ID_CANVAS,
	    ID_GLVIEW,
	    ID_CLRUPDATE
	    };

    long onCmdAbout(FXObject*, FXSelector, void*);
    long onCmdOpen(FXObject*, FXSelector, void*);
    long onCmdSaveAs(FXObject*, FXSelector, void*);
    long onCmdRegExec(FXObject*, FXSelector, void*);

protected:
    CPSRegWin(){} // default

private:
    FXToolBar    *m_toolb;
    FXMenuBar    *m_menub;
    FXStatusBar  *m_statb;
    FXMenuPane   *m_filem;
    FXMenuPane   *m_viewm;
    FXMenuPane   *m_regm;
    FXMenuPane   *m_helpm;
    FXTreeList   *m_treel;
    FXGLVisual   *m_glvisual;
    FXGLCanvas   *m_glcanvas;
    CKFXGLView   *m_glview;
    
    int m_nNObjs;

    // The 3D objects should be stored in CPSReg
    CPSReg*       m_ppsr;

    // Manage the Tree List
    std::vector<FXTreeItem*> m_vfxtis;

    // Args for CPSReg
    int m_nNSteps, m_nNRKSteps, m_nNLMSteps;
    float m_fMESigma, m_fNThresh, m_fTDSearch, m_fCGSearch, m_fBack;
    int m_nBCheck, m_nIEval, m_nMultiNP, m_nPair, m_nDump;

    int m_nCnt;

    int getSprtFilenames(std::string& strFNames, 
			 std::vector<std::string>& vstrFNames);
    int getSaveName(FXString& fxsFName, std::string& strFName);

};
    


#endif // _PSREGWIN_H_
