/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  Pose.cpp
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#include "KDefs.h"
#include <assert.h>
#include "Pose.h"
#include "Utils.h"

CPose::CPose()
{
    m_pose = gsl_vector_alloc(7);
    initialize();

}

CPose::CPose(const CPose& cp)
{
    m_pose = gsl_vector_alloc(7);
    gsl_vector_memcpy(m_pose, cp.getPose());
}
    
CPose::~CPose()
{
    gsl_vector_free(m_pose);
}

void CPose::getMatrix(gsl_matrix* M) const
{
    assertM4x4(M);
    gsl_matrix_set_identity(M);
    getRotMatrix(M);
    for(int i = 0; i < 3; i++) gsl_matrix_set(M, i, 3, gsl_vector_get(m_pose, i));
}

void CPose::getRotMatrix(gsl_matrix* M) const 
{
    assertM4x4(M);
    gsl_matrix_set_identity(M);
    gsl_vector* v4 = gsl_vector_alloc(4);
    for(int i = 0; i < 4; i++) gsl_vector_set(v4, i, gsl_vector_get(m_pose, i+3));
    
    Utils::gsl_vector_normalize(v4);
    double d0, d1, d2, d3;
    d0 = gsl_vector_get(v4, 0);
    d1 = gsl_vector_get(v4, 1);
    d2 = gsl_vector_get(v4, 2);
    d3 = gsl_vector_get(v4, 3);
    gsl_matrix_set(M, 0, 0, 1.0-2.0*(d1*d1+d2*d2));
    gsl_matrix_set(M, 0, 1, 2.0*(d0*d1-d3*d2));
    gsl_matrix_set(M, 0, 2, 2.0*(d0*d2+d3*d1));
    gsl_matrix_set(M, 1, 0, 2.0*(d0*d1+d3*d2));
    gsl_matrix_set(M, 1, 1, 1.0-2.0*(d0*d0+d2*d2));
    gsl_matrix_set(M, 1, 2, 2.0*(d1*d2-d3*d0));
    gsl_matrix_set(M, 2, 0, 2.0*(d0*d2-d3*d1));
    gsl_matrix_set(M, 2, 1, 2.0*(d1*d2+d3*d0));
    gsl_matrix_set(M, 2, 2, 1.0-2.0*(d0*d0+d1*d1));

    gsl_vector_free(v4);
}
