/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  Pose.h
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#ifndef _POSE_H_
#define _POSE_H_

#include <assert.h>
#include <math.h>
#include <iostream>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>



class CPose
{
public:
    CPose();
    CPose(const CPose& cp);
    virtual ~CPose();

    const gsl_vector* getPose() const { return m_pose; }
    void getMatrix(gsl_matrix* M) const;
    void getRotMatrix(gsl_matrix* M) const;

    inline void setPose(gsl_vector* v);     
    inline void setPose(gsl_matrix* m);
    inline void initialize();
    inline void setUnit();
    inline void addVector(const gsl_vector* v);
    inline void addVector(const int nF, const int nT, const gsl_vector* v);
    inline void scale(const double dScale);
    inline void scale(const int nF, const int nT, const double dScale);
    inline double dot() const;
    inline double dot(const int nF, const int nT) const;
    inline void print() const; 
    inline bool checkZero();

    CPose& operator=(const CPose& rhs){
	if(this == &rhs) return *this;
	gsl_vector_memcpy(this->m_pose, rhs.m_pose);
	return *this;
    }

    CPose operator+(const CPose& rhs) const {
	CPose result;
	gsl_vector_memcpy(result.m_pose, this->m_pose);
	gsl_vector_add(result.m_pose, rhs.m_pose);
	return result;
    }
	
    CPose operator*(const double rhs) const {
	CPose result;
	gsl_vector_memcpy(result.m_pose, this->m_pose);
	gsl_vector_scale(result.m_pose, rhs);
	return result;
    }


private:
    gsl_vector* m_pose; 

    void assertM4x4(const gsl_matrix* M) const {
	assert(M->size1 == 4 && M->size2 == 4);
    }
    void assertV7(const gsl_vector* V) const {
	assert(V->size == 7);
    }
};

#include "Pose.inl"
 
#endif // _POSE_H_
