/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  Pose.inl
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */
// inline functions of CPose

inline void CPose::initialize()
{
    gsl_vector_set_zero(m_pose);
}

inline void CPose::print() const
{
    for(int i = 0; i < 7; i++){
	std::cout << gsl_vector_get(m_pose, i) << " ";
    }
    std::cout << std::endl;
}

inline bool CPose::checkZero()
{
    bool bFlag(false);
    for(int i = 0; i < 7; i++){
	if(fabs(gsl_vector_get(m_pose, i)) < KEPSILON){
	    bFlag = true;
	}
    }

    if(bFlag) this->setUnit();
    return bFlag;
}

inline void CPose::setPose(gsl_matrix* m)
{
    assertM4x4(m);
    std::cout << "CPose::setPose not implemented yet" << std::endl;
}

inline void CPose::setPose(gsl_vector* v){
    assertV7(v);
    gsl_vector_memcpy(m_pose, v);
}

inline void CPose::setUnit(){
    gsl_vector_set(m_pose, 6, 1.0);
}

inline void CPose::addVector(const gsl_vector* v){
    assertV7(v);
    gsl_vector_add(m_pose, v);
}

inline void CPose::addVector(const int nF, const int nT, const gsl_vector* v){
    for(int i = nF; i <= nT; i++){
	gsl_vector_set(m_pose, i, gsl_vector_get(m_pose, i)+gsl_vector_get(v, i));
    }
}

inline void CPose::scale(const double dScale){
    gsl_vector_scale(m_pose, dScale);
}

inline void CPose::scale(const int nF, const int nT, const double dScale){
    for(int i = nF; i <= nT; i++){
	gsl_vector_set(m_pose, i, gsl_vector_get(m_pose, i)*dScale);
    }
}

inline double CPose::dot() const {
    return dot(0, 6);
}

inline double CPose::dot(const int nF, const int nT) const {
    double dsum(0.0);
    for(int i = nF; i <= nT; i++){
	dsum += gsl_vector_get(m_pose, i)*gsl_vector_get(m_pose, i);
    }
    return dsum;
}    
