/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  RPoint.cpp
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#include "KDefs.h"
#include "RPoint.h"
#include "Utils.h"

CRPoint::CRPoint()
{
    m_vC = gsl_vector_alloc(4);
    gsl_vector_set_zero(m_vC);
    gsl_vector_set(m_vC, 3, 1.0);

    m_vN = gsl_vector_alloc(4);
    gsl_vector_set_zero(m_vN);
    gsl_vector_set(m_vN, 3, 1.0);
}

CRPoint::~CRPoint()
{
    gsl_vector_free(m_vC);
    gsl_vector_free(m_vN);
}

void CRPoint::transform(const CPose& cp)
{
    gsl_matrix* gmat;
    gmat = gsl_matrix_alloc(4, 4);
    cp.getMatrix(gmat);

    Utils::gsl_matrix_vector_mult(gmat, m_vC, m_vC);
    
    cp.getRotMatrix(gmat);
    Utils::gsl_matrix_vector_mult(gmat, m_vN, m_vN);

    gsl_matrix_free(gmat);
}
