/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  RegExecDlg.cpp
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#include "RegExecDlg.h"

#define M_WIDTH 15

FXIMPLEMENT(CRegExecDlg, FXDialogBox, NULL, 0)

CRegExecDlg::CRegExecDlg(FXWindow* owner, int nNSteps, int nNRKSteps, int nNLMSteps, 
			 float fMESigma, float fNThresh,
			 float fTDSearch, float fCGSearch,
			 float fBack, 
			 int nBCheck, int nIEval, int nMultiNP, 
			 int nPair, int nDump)
    : FXDialogBox(owner, "Registration Params", DECOR_TITLE|DECOR_BORDER),
      m_nNSteps(nNSteps), m_nNRKSteps(nNRKSteps), m_nNLMSteps(nNLMSteps), 
      m_fMESigma(fMESigma), m_fNThresh(fNThresh), m_fTDSearch(fTDSearch), m_fCGSearch(fCGSearch), m_fBack(fBack),
      m_nBCheck(nBCheck), m_nIEval(nIEval), m_nMultiNP(nMultiNP),
      m_nPair(nPair), m_nDump(nDump)
{
    m_dtNSteps.connect(m_nNSteps);
    m_dtNRKSteps.connect(m_nNRKSteps);
    m_dtNLMSteps.connect(m_nNLMSteps);
    m_dtMESigma.connect(m_fMESigma);
    m_dtNThresh.connect(m_fNThresh);
    m_dtTDSearch.connect(m_fTDSearch);
    m_dtCGSearch.connect(m_fCGSearch);
    m_dtBCheck.connect(m_nBCheck);
    m_dtIEval.connect(m_nIEval);
    m_dtMultiNP.connect(m_nMultiNP);
    m_dtPair.connect(m_nPair);
    m_dtBack.connect(m_fBack);
    m_dtDump.connect(m_nDump);

    // Window layout
    m_mainfrm = new FXHorizontalFrame(this, \
				      LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);

    m_matrix = new FXMatrix(m_mainfrm, 2, \
			    MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    new FXLabel(m_matrix, "&Iterations", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtNSteps, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_INTEGER|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXLabel(m_matrix, "&Inner Loop", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtNRKSteps, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_INTEGER|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXLabel(m_matrix, "&LM Loop", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtNLMSteps, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_INTEGER|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXLabel(m_matrix, "&MEst Sigma", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtMESigma, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_REAL|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXLabel(m_matrix, "&Normal Threshhold", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtNThresh, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_REAL|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXLabel(m_matrix, "&TotalDiff Search Range", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtTDSearch, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_REAL|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);


    new FXLabel(m_matrix, "&Gradient Search Range", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtCGSearch, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_REAL|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXLabel(m_matrix, "&Backface Culling Percentage", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtBack, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_REAL|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXCheckButton(m_matrix, "Boundary Check", &m_dtBCheck, FXDataTarget::ID_VALUE);

    new FXCheckButton(m_matrix, "Eval Intensity", &m_dtIEval, FXDataTarget::ID_VALUE);

    new FXLabel(m_matrix, "&Intensity Search Range", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
    new FXTextField(m_matrix, M_WIDTH, &m_dtMultiNP, FXDataTarget::ID_VALUE, \
		    TEXTFIELD_INTEGER|JUSTIFY_RIGHT|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXCheckButton(m_matrix, "Pairwise", &m_dtPair, FXDataTarget::ID_VALUE);
    new FXLabel(m_matrix, " ", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

    new FXCheckButton(m_matrix, "Dump Images", &m_dtDump, FXDataTarget::ID_VALUE);
    new FXLabel(m_matrix, " ", NULL, \
		LAYOUT_CENTER_Y|LAYOUT_CENTER_X|JUSTIFY_RIGHT|LAYOUT_FILL_ROW|LAYOUT_FILL_X);

		      

    new FXButton(m_matrix, "&Run", NULL, this, ID_ACCEPT, \
		 FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_Y|LAYOUT_FILL_X);
    new FXButton(m_matrix, "&Cancel", NULL, this, ID_CANCEL, \
		 FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_Y|LAYOUT_FILL_X);
 
				      

}

CRegExecDlg::~CRegExecDlg()
{
    //delete m_mainfrm;
    //delete m_matrix;
}

