/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  RegExecDlg.h
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#ifndef _REGEXECDLG_H_
#define _REGEXECDLG_H_

#include <fox/fx.h>

class CRegExecDlg : public FXDialogBox
{
    FXDECLARE(CRegExecDlg)

public:
    CRegExecDlg(FXWindow* owner, 
		int nNSteps = 5, int nNRKSteps = 3, int nNLMSteps = 5, 
		float fMESigma = 1.0, float fNThresh = -1.0, 
		float fTDSearch = .01, float fCGSearch = .01,
		float fBack = .1, 
		int bBCheck = 0, int bIEval = 0, int nMultiNP = 10, 
		int nPair = 0, int nDump = 1);
    virtual ~CRegExecDlg();
    int getNSteps() const { return m_nNSteps; }
    int getNRKSteps() const { return m_nNRKSteps; }
    int getNLMSteps() const { return m_nNLMSteps; }
    float getMESigma() const { return m_fMESigma; }
    float getNThresh() const { return m_fNThresh; }
    float getTDSearch() const { return m_fTDSearch; }
    float getCGSearch() const { return m_fCGSearch; }
    float getBack() const { return m_fBack; }
    int getBCheck() const { return m_nBCheck; }
    int getIEval() const { return m_nIEval; }
    int getMultiNP() const { return m_nMultiNP; }
    int getPair() const { return m_nPair; }
    int getDump() const { return m_nDump; }

protected:
    FXHorizontalFrame *m_mainfrm;
    FXHorizontalFrame *m_buttfrm;
    FXMatrix          *m_matrix;
    
    FXDataTarget       m_dtNSteps;
    FXDataTarget       m_dtNRKSteps;
    FXDataTarget       m_dtNLMSteps;
    FXDataTarget       m_dtMESigma;
    FXDataTarget       m_dtNThresh;
    FXDataTarget       m_dtTDSearch;
    FXDataTarget       m_dtCGSearch;
    FXDataTarget       m_dtBCheck;
    FXDataTarget       m_dtIEval;
    FXDataTarget       m_dtMultiNP;
    FXDataTarget       m_dtPair;
    FXDataTarget       m_dtBack;
    FXDataTarget       m_dtDump;

    
private:
    CRegExecDlg(){}
    int m_nNSteps, m_nNRKSteps, m_nNLMSteps;
    float m_fMESigma, m_fNThresh, m_fTDSearch, m_fCGSearch, m_fBack;
    int m_nBCheck, m_nIEval, m_nMultiNP, m_nPair, m_nDump;

};

#endif // _REGEXECDLG_H_
