/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  RegKernel.inl
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */
// inline functions of CRegKernel

inline void CRegKernel::getRPfromCntnt(CRPoint& crp, const CNTNT* pcntnt, const unsigned long ulid) const
{
    crp.setCoord(pcntnt->data[ulid*3], pcntnt->data[ulid*3+1], pcntnt->data[ulid*3+2]);
    crp.setNormal(pcntnt->normal[ulid*3], pcntnt->normal[ulid*3+1], pcntnt->normal[ulid*3+2]);
    crp.setInten(pcntnt->inten[ulid]);
}

inline bool CRegKernel::NormalCheck(const CRPoint& crp, const CNTNT* pcntnt, const int nid, const float fthresh) const
{
    if((gsl_vector_get(crp.getNormal(), 0)*pcntnt->normal[nid*3]+
	gsl_vector_get(crp.getNormal(), 1)*pcntnt->normal[nid*3+1]+
	gsl_vector_get(crp.getNormal(), 2)*pcntnt->normal[nid*3+2]) > fthresh){
	return true;
    }else{
	return false;
    }
}

inline void CRegKernel::normalizeGradient(CPose& cpG) const
{
    double fs = 1.0/m_fMScale;
    double fs2 = fs*fs;
    
    cpG.scale(0, 2, fs);
    cpG.scale(3, 5, fs2);
}

inline float CRegKernel::Lorentzian(float fdist) const
{
    return (float)log(1.0+0.5*fdist/m_fMESigma2);
}

inline float CRegKernel::DLorentzian(float fdist) const
{
    return ((float)1.0/(2.0*m_fMESigma2+fdist));
}

