/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  Utils.cpp
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */

#include "KDefs.h"
#include <assert.h>
#include "Utils.h"

void Utils::dumpPBytesAsPPM(const unsigned char* pub, const int nWidth, const int nHeight, const char* pcFName)
{
    std::ofstream ofs;
    ofs.open(pcFName, std::ios::out|std::ios::binary);
    
    ofs << "P6" << std::endl;
    ofs << "# binary ppm file created by PSReg" << std::endl;
    ofs << nWidth << " " << nHeight << std::endl;
    ofs << "255" << std::endl;
    
    int i;
    for(int y = nHeight-1; y >= 0; y--){
	for(int x = 0; x < nWidth; x++){
	    i = (y*nWidth+x)*3;
	    ofs << pub[i] << pub[i+1] << pub[i+2];
	}
    }
    ofs.close();

}


int Utils::countNonBlackPixs(const unsigned char* pub, const int nWidth, const int nHeight)
{
    int nNB(0), i;
    for(int x = 0; x < nWidth; x++){
	for(int y = 0; y < nHeight; y++){
	    i = 3*(x*nHeight+y);
	    if(pub[i] > 0 || pub[i+1] > 0 || pub[i+2] > 0){
		nNB++;
	    }
	}
    }

    return nNB;
}
