/**************************************************************************
 *                                                                        *
 *                       This file is part of PSReg                       *
 *                                                                        *
 *          Copyright (C) 2002 Ko Nishino. All Rights Reserved.           *
 *                                                                        *
 *  Permission to use this software for educational and research purposes *
 * is hereby granted without fee. All other rights, including the rights  *
 * to distribute, modify, or sell the software, are explicitly reserved   *
 * by the author. Please contact the author if you wish to make any       *
 * modifications or additions to the software.                            *
 *                                                                        *
 *  This software is provided "as is" and without warranty of any kind,   *
 * express, implied or otherwise, including without limitation, any       *
 * warranty of merchantability or fitness for a particular purpose.       *
 *                                                                        *
 **************************************************************************/
/*  
 *  Utils.h
 *          Ko Nishino 
 *               kon@cvl.iis.u-tokyo.ac.jp
 *               http://www.cvl.iis.u-tokyo.ac.jp/~kon/
 */


#ifndef _UTILS_H_
#define _UTILS_H_

#include <assert.h>
#include <math.h>
#include <string>
#include <iostream>
#include <fstream>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>

namespace Utils{
    inline void gsl_vector_normalize(gsl_vector* v);
    inline void gsl_vector3_cross(const gsl_vector* vl, const gsl_vector* vr, gsl_vector* v);
    inline double gsl_vector_dot(const int nF, const int nT, const gsl_vector* v);
    inline void gsl_matrix_diff(gsl_matrix* oldm, gsl_matrix* newm, gsl_matrix* m);
    inline void gsl_matrix_vector_mult(gsl_matrix* m, gsl_vector* v, gsl_vector* vr);
    inline void gsl_matrix4x4_mult(gsl_matrix* ml, gsl_matrix* mr, gsl_matrix* m);
    inline void gsl_matrix4x4_from_pfloat(gsl_matrix* m, const float* pglf, const bool btrans);
    inline void gsl_matrix4x4_to_pfloat(const gsl_matrix* m, float* pglf, const bool btrans);

    inline void Intensity2RGB(float* pC, float fInten, float* pBaseC);

    int countNonBlackPixs(const unsigned char* pub, const int nWidth, const int nHeight);
    void dumpPBytesAsPPM(const unsigned char* pub, const int nWidth, const int nHeight, const char* pcFName);
};

#include "Utils.inl"

#endif // _UTILS_H_
